@@ -59,7 +59,7 @@ struct FileDateTracker {
 #ifdef G_THREADS_ENABLED
 
 /* We lock this mutex whenever we modify global state in this module.  */
-G_LOCK_DEFINE_STATIC (mime_mutex);
+G_LOCK_DEFINE (gnome_vfs_mime_mutex);
 
 #endif /* G_LOCK_DEFINE_STATIC */
 
@@ -71,27 +71,27 @@ G_LOCK_DEFINE_STATIC (mime_mutex);
  * operating system automatically cleans up resources when exiting.
  *
  * Unload the MIME database from memory.
- **/
+ */
 
 void
 gnome_vfs_mime_shutdown (void)
 {
-	G_LOCK (mime_mutex);
+	G_LOCK (gnome_vfs_mime_mutex);
 
 	xdg_mime_shutdown ();
 
-	G_UNLOCK (mime_mutex);
+	G_UNLOCK (gnome_vfs_mime_mutex);
 }
 
 /**
  * gnome_vfs_mime_type_from_name_or_default:
- * @filename: A filename (the file does not necesarily exist).
- * @defaultv: A default value to be returned if no match is found
+ * @filename: a filename (the file does not necessarily exist).
+ * @defaultv: a default value to be returned if no match is found.
  *
  * This routine tries to determine the mime-type of the filename
  * only by looking at the filename from the GNOME database of mime-types.
  *
- * Returns the mime-type of the @filename.  If no value could be
+ * Returns: the mime-type of the @filename.  If no value could be
  * determined, it will return @defaultv.
  */
 const char *
@@ -121,9 +121,9 @@ gnome_vfs_mime_type_from_name_or_default (const char *filename, const char *defa
 		separator = filename;
 	}
 
-	G_LOCK (mime_mutex);
+	G_LOCK (gnome_vfs_mime_mutex);
 	mime_type = xdg_mime_get_mime_type_from_file_name (separator);
-	G_UNLOCK (mime_mutex);
+	G_UNLOCK (gnome_vfs_mime_mutex);
 
 	if (mime_type)
 		return mime_type;
@@ -132,17 +132,81 @@ gnome_vfs_mime_type_from_name_or_default (const char *filename, const char *defa
 }
 
 /**
- * gnome_vfs_mime_type_from_name:
- * @filename: A filename (the file does not necessarily exist).
+ * gnome_vfs_get_mime_type_for_name:
+ * @filename: a filename.
  *
- * Determined the mime type for @filename.
+ * Determine the mime type for @filename. The file @filename may
+ * not exist, this function does not access the actual file.
+ * If the mime-type cannot be determined, %GNOME_VFS_MIME_TYPE_UNKNOWN
+ * is returned.
+ * 
+ * Returns: the mime-type for this filename or
+ * %GNOME_VFS_MIME_TYPE_UNKNOWN if mime-type could not be determined.
  *
- * Returns the mime-type for this filename.
+ * Since: 2.14
  */
 const char *
-gnome_vfs_mime_type_from_name (const gchar * filename)
+gnome_vfs_get_mime_type_for_name (const char *filename)
 {
-	return gnome_vfs_mime_type_from_name_or_default (filename, GNOME_VFS_MIME_TYPE_UNKNOWN);
+	return gnome_vfs_mime_type_from_name_or_default (filename,
+							 GNOME_VFS_MIME_TYPE_UNKNOWN);
+}
+
+/**
+ * gnome_vfs_mime_type_from_name:
+ * @filename: a filename (the file does not necessarily exist).
+ *
+ * Determine the mime type for @filename.
+ *
+ * Deprecated: This function is deprecated, use
+ * gnome_vfs_get_mime_type_for_name instead.
+ *
+ * Returns: the mime-type for this filename. Will return
+ * %GNOME_VFS_MIME_TYPE_UNKNOWN if mime-type could not be found.
+ */
+const char *
+gnome_vfs_mime_type_from_name (const gchar *filename)
+{
+	return gnome_vfs_get_mime_type_for_name (filename);
+}
+
+/**
+ * gnome_vfs_get_mime_type_for_name_and_data:
+ * @filename: a filename.
+ * @data: a pointer to the data in the memory
+ * @data_size: the size of @data
+ *
+ * Determine the mime-type for @filename and @data. This function tries
+ * to be smart (e.g. mime subclassing) about returning the right mime-type 
+ * by looking at both the @data and the @filename. The file will not be 
+ * accessed.
+ * If the mime-type cannot be determined, %GNOME_VFS_MIME_TYPE_UNKNOWN
+ * is returned.
+ * 
+ * Returns: the mime-type for this filename or
+ * %GNOME_VFS_MIME_TYPE_UNKNOWN if mime-type could not be determined.
+ *
+ * Since: 2.14
+ */
+const char *
+gnome_vfs_get_mime_type_for_name_and_data (const char    *filename,
+					   gconstpointer  data,
+					   gssize         data_size)
+{
+	GnomeVFSMimeSniffBuffer *sniff_buffer;
+	const char *mime_type;
+	
+	sniff_buffer = gnome_vfs_mime_sniff_buffer_new_from_existing_data (data,
+									   data_size);
+
+	
+	mime_type = _gnome_vfs_get_mime_type_internal (sniff_buffer,
+						       filename,
+						       TRUE);
+
+	gnome_vfs_mime_sniff_buffer_free (sniff_buffer);
+
+	return mime_type;
 }
 
 static const char *
@@ -173,9 +237,9 @@ _gnome_vfs_read_mime_from_buffer (GnomeVFSMimeSniffBuffer *buffer)
 	GnomeVFSResult result = GNOME_VFS_OK;
 	const char *mime_type;
 
-	G_LOCK (mime_mutex);
+	G_LOCK (gnome_vfs_mime_mutex);
 	max_extents = xdg_mime_get_max_buffer_extents ();
-	G_UNLOCK (mime_mutex);
+	G_UNLOCK (gnome_vfs_mime_mutex);
 	max_extents = CLAMP (max_extents, 0, MAX_SNIFF_BUFFER_ALLOWED);
 
 	if (!buffer->read_whole_file) {
@@ -184,18 +248,20 @@ _gnome_vfs_read_mime_from_buffer (GnomeVFSMimeSniffBuffer *buffer)
 	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
 		return NULL;
 	}
-	G_LOCK (mime_mutex);
+	G_LOCK (gnome_vfs_mime_mutex);
 
 	mime_type = xdg_mime_get_mime_type_for_data (buffer->buffer, buffer->buffer_length);
 
-	G_UNLOCK (mime_mutex);
+	G_UNLOCK (gnome_vfs_mime_mutex);
 
 	return mime_type;
 	
 }
 
 const char *
-_gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *file_name, gboolean use_suffix)
+_gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer,
+				   const char              *file_name,
+				   gboolean                 use_suffix)
 {
 	const char *result;
 	const char *fn_result;
@@ -229,9 +295,12 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
 					result = fn_result;
 				}
 				
-			} else if (fn_result && fn_result != XDG_MIME_TYPE_UNKNOWN &&
-				   xdg_mime_mime_type_subclass (fn_result, result)) {
-				result = fn_result;
+			} else if (fn_result && fn_result != XDG_MIME_TYPE_UNKNOWN) {
+				G_LOCK (gnome_vfs_mime_mutex);
+				if (xdg_mime_mime_type_subclass (fn_result, result)) {
+					result = fn_result;
+				}
+				G_UNLOCK (gnome_vfs_mime_mutex);
 			}
 			
 			return result;
@@ -243,9 +312,13 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
 				 * accurate source of type information BUT _only_ if
 				 * the extension is a subtype of text/plain.
 				 */
-				if ((fn_result != NULL) && (fn_result != XDG_MIME_TYPE_UNKNOWN) &&
-				    xdg_mime_mime_type_subclass (fn_result, "text/plain")) {
-					return fn_result;
+				if ((fn_result != NULL) && (fn_result != XDG_MIME_TYPE_UNKNOWN)) {
+					G_LOCK (gnome_vfs_mime_mutex);
+					if (xdg_mime_mime_type_subclass (fn_result, "text/plain")) {
+						G_UNLOCK (gnome_vfs_mime_mutex);
+						return fn_result;
+					}
+					G_UNLOCK (gnome_vfs_mime_mutex);
 				}
 
 				/* Didn't find an extension match, assume plain text. */
@@ -274,16 +347,16 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
  * gnome_vfs_get_mime_type_common:
  * @uri: a real file or a non-existent uri.
  *
- * Tries to guess the mime type of the file represented by @uir.
+ * Tries to guess the mime type of the file represented by @uri.
  * Favors using the file data to the @uri extension.
- * Handles passing @uri of a non-existent file by falling back
- * on returning a type based on the extension.
+ * Handles @uri of a non-existent file by falling back
+ * on returning a type based on the extension. If cant find the mime-type based on the 
+ * extension also then returns 'application/octet-stream'.
  *
  * FIXME: This function will not necessarily return the same mime type as doing a
  * get file info on the text uri.
  *
- * Returns: the mime-type for this uri.
- * 
+ * Returns: the mime-type for @uri.
  */
 const char *
 gnome_vfs_get_mime_type_common (GnomeVFSURI *uri)
@@ -337,11 +410,18 @@ static GnomeVFSResult
 file_read_binder (gpointer context, gpointer buffer, 
 		  GnomeVFSFileSize bytes, GnomeVFSFileSize *bytes_read)
 {
-	FILE *file = (FILE *)context;	
+	FILE *file = (FILE *)context;
+	
 	*bytes_read = fread (buffer, 1, bytes, file);
-	if (*bytes_read < 0) {
-		*bytes_read = 0;
-		return gnome_vfs_result_from_errno ();
+	
+	/* short read, check eof */
+	if (*bytes_read < bytes) {
+	
+		if (feof (file)) {
+			return GNOME_VFS_ERROR_EOF;
+		} else {
+			return gnome_vfs_result_from_errno ();
+		} 
 	}
 
 	return GNOME_VFS_OK;
@@ -415,17 +495,17 @@ gnome_vfs_get_file_mime_type_internal (const char *path, const struct stat *opti
 
 /**
  * gnome_vfs_get_file_mime_type_fast:
- * @path: a path of a file.
+ * @path: path of the file whose mime type is to be found out.
  * @optional_stat_info: optional stat buffer.
  *
  * Tries to guess the mime type of the file represented by @path.
- * If It uses extention/name detection first, and if that fails
+ * It uses extention/name detection first, and if that fails
  * it falls back to mime-magic based lookup. This is faster
- * than always doing mime-magic, but doesn't always produce
+ * than always doing mime-magic but doesn't always produce
  * the right answer, so for important decisions 
- * you should use gnome_vfs_get_file_mime_type.
+ * you should use gnome_vfs_get_file_mime_type().
  *
- * Returns: the mime-type for this path
+ * Returns: the mime-type for file at @path.
  */
 const char *
 gnome_vfs_get_file_mime_type_fast (const char *path, const struct stat *optional_stat_info)
@@ -436,19 +516,19 @@ gnome_vfs_get_file_mime_type_fast (const char *path, const struct stat *optional
 
 /**
  * gnome_vfs_get_file_mime_type:
- * @path: a path of a file.
+ * @path: path of a file whose mime type is to be found out.
  * @optional_stat_info: optional stat buffer.
  * @suffix_only: whether or not to do a magic-based lookup.
  *
  * Tries to guess the mime type of the file represented by @path.
  * If @suffix_only is false, uses the mime-magic based lookup first.
- * Handles passing @path of a non-existent file by falling back
+ * Handles @path of a non-existent file by falling back
  * on returning a type based on the extension.
  *
  * If you need a faster, less accurate version, use
- * @gnome_vfs_get_file_mime_type_fast.
+ * gnome_vfs_get_file_mime_type_fast().
  *
- * Returns: the mime-type for this path
+ * Returns: the mime-type for file at @path.
  */
 const char *
 gnome_vfs_get_file_mime_type (const char *path, const struct stat *optional_stat_info,
@@ -459,13 +539,13 @@ gnome_vfs_get_file_mime_type (const char *path, const struct stat *optional_stat
 
 /**
  * gnome_vfs_get_mime_type_from_uri:
- * @uri: A file uri.
+ * @uri: a file uri.
  *
  * Tries to guess the mime type of the file @uri by
  * checking the file name extension. Works on non-existent
  * files.
  *
- * Returns the mime-type for this filename.
+ * Returns: the mime-type for file at @uri.
  */
 const char *
 gnome_vfs_get_mime_type_from_uri (GnomeVFSURI *uri)
@@ -483,14 +563,14 @@ gnome_vfs_get_mime_type_from_uri (GnomeVFSURI *uri)
 
 /**
  * gnome_vfs_get_mime_type_from_file_data:
- * @uri: A file uri.
+ * @uri: a file uri.
  *
  * Tries to guess the mime type of the file @uri by
- * checking the file data using the magic patterns. Does not handle text files properly
+ * checking the file data using the magic patterns. Does not handle text files properly.
  *
  * Deprecated:
  *
- * Returns the mime-type for this filename.
+ * Returns: the mime-type for this filename.
  */
 const char *
 gnome_vfs_get_mime_type_from_file_data (GnomeVFSURI *uri)
@@ -516,13 +596,13 @@ gnome_vfs_get_mime_type_from_file_data (GnomeVFSURI *uri)
 
 /**
  * gnome_vfs_get_mime_type_for_data:
- * @data: A pointer to data in memory.
- * @data_size: Size of the data.
+ * @data: a pointer to data in memory.
+ * @data_size: size of the data.
  *
  * Tries to guess the mime type of the data in @data
  * using the magic patterns.
  *
- * Returns the mime-type for this filename.
+ * Returns: the mime-type for @data.
  */
 const char *
 gnome_vfs_get_mime_type_for_data (gconstpointer data, int data_size)
@@ -577,7 +657,10 @@ extract_prefix_add_suffix (const char *string, const char *separator, const char
         return result;
 }
 
-/* Returns the supertype for a mime type. Note that if called
+/* gnome_vfs_get_supertype_from_mime_type:
+ * @mime_type: mimetype to get the supertype for.
+ *
+ * Returns: the supertype for @mime type. Note that if called
  * on a supertype it will return a copy of the supertype.
  */
 char *
@@ -592,14 +675,14 @@ gnome_vfs_get_supertype_from_mime_type (const char *mime_type)
 
 /**
  * gnome_vfs_mime_type_is_equal:
- * @a: A const char * containing a mime type, e.g. "image/png"
- * @b: A const char * containing a mime type, e.g. "image/png"
+ * @a: a const char * containing a mime type, e.g. "image/png".
+ * @b: a const char * containing a mime type, e.g. "image/png".
  * 
  * Compares two mime types to determine if they are equivalent.  They are
  * equivalent if and only if they refer to the same mime type.
  * 
- * Return value: %TRUE, if a and b are equivalent mime types
- **/
+ * Return value: %TRUE, if a and b are equivalent mime types.
+ */
 gboolean
 gnome_vfs_mime_type_is_equal (const char *a,
 			      const char *b)
@@ -607,33 +690,35 @@ gnome_vfs_mime_type_is_equal (const char *a,
 	g_return_val_if_fail (a != NULL, FALSE);
 	g_return_val_if_fail (b != NULL, FALSE);
 
-	G_LOCK (mime_mutex);
+	G_LOCK (gnome_vfs_mime_mutex);
 	xdg_mime_mime_type_equal (a, b);
-	G_UNLOCK (mime_mutex);
+	G_UNLOCK (gnome_vfs_mime_mutex);
 
 	return FALSE;
 }
 
 /**
  * gnome_vfs_mime_type_get_equivalence:
- * @mime_type: A const char * containing a mime type, e.g. "image/png"
- * @base_mime_type: A const char * containing either a mime type or a subtype.
+ * @mime_type: a const char * containing a mime type, e.g. "image/png".
+ * @base_mime_type: a const char * containing either a mime type or a base mime type.
  * 
- * Compares @mime_type to @base_mime_type.  There are a three possible
- * relationships between the two strings.  If they are identical and @mime_type
- * is the same as @base_mime_type, then #GNOME_VFS_MIME_IDENTICAL is returned.
- * This would be the case if "audio/midi" and "audio/x-midi" are passed in.
+ * Compares @mime_type to @base_mime_type.  There are three possible
+ * relationships between the two strings.
  *
- * If @base_mime_type is a parent type of @mime_type, then
- * #GNOME_VFS_MIME_PARENT is returned.  As an example, "text/plain" is a parent
+ * a) If they are identical and @mime_type is the same as @base_mime_type,
+ *  then %GNOME_VFS_MIME_IDENTICAL is returned. This would be the case if
+ *  "audio/midi" and "audio/x-midi" are passed in.
+ *
+ * b) If @base_mime_type is a parent type of @mime_type, then
+ * %GNOME_VFS_MIME_PARENT is returned.  As an example, "text/plain" is a parent
  * of "text/rss", "image" is a parent of "image/png", and
  * "application/octet-stream" is a parent of almost all types.
  *
- * Finally, if the two mime types are unrelated, than #GNOME_VFS_MIME_UNRELATED
+ * c) Finally, if the two mime types are unrelated, than %GNOME_VFS_MIME_UNRELATED
  * is returned.
  * 
- * Return value:
- **/
+ * Returns: value 0,1 or 2 indicating the relationship between @mime_type and @base_mime_type.
+ */
 GnomeVFSMimeEquivalence
 gnome_vfs_mime_type_get_equivalence (const char *mime_type,
 				     const char *base_mime_type)
@@ -644,12 +729,12 @@ gnome_vfs_mime_type_get_equivalence (const char *mime_type,
 	if (gnome_vfs_mime_type_is_equal (mime_type, base_mime_type)) {
 		return GNOME_VFS_MIME_IDENTICAL;
 	} else {
-		G_LOCK (mime_mutex);
+		G_LOCK (gnome_vfs_mime_mutex);
 		if (xdg_mime_mime_type_subclass (mime_type, base_mime_type)) {
-			G_UNLOCK (mime_mutex);
+			G_UNLOCK (gnome_vfs_mime_mutex);
 			return GNOME_VFS_MIME_PARENT;
 		}
-		G_UNLOCK (mime_mutex);
+		G_UNLOCK (gnome_vfs_mime_mutex);
 	}
 
 	return GNOME_VFS_MIME_UNRELATED;
@@ -784,15 +869,15 @@ _gnome_vfs_file_date_tracker_date_has_changed (FileDateTracker *tracker)
 
 /**
  * gnome_vfs_get_mime_type:
- * @text_uri: URI of the file for which to get the mime type
- * 
+ * @text_uri: path of the file for which to get the mime type.
+ *
  * Determine the mime type of @text_uri. The mime type is determined
  * in the same way as by gnome_vfs_get_file_info(). This is meant as
- * a convenience function for times when you only want the mime type.
- * 
- * Return value: The mime type, or NULL if there is an error reading 
+ * a convenient function for times when you only want the mime type.
+ *
+ * Return value: The mime type, or %NULL if there is an error reading
  * the file.
- **/
+ */
 char *
 gnome_vfs_get_mime_type (const char *text_uri)
 {
@@ -814,27 +899,59 @@ gnome_vfs_get_mime_type (const char *text_uri)
 	return mime_type;
 }
 
-/* This is private due to the feature freeze, maybe it should be public */
+/**
+ * gnome_vfs_get_slow_mime_type:
+ * @text_uri: URI of the file for which to get the mime type
+ * 
+ * Determine the mime type of @text_uri. The mime type is determined
+ * in the same way as by gnome_vfs_get_file_info(). This is meant as
+ * a convenience function for times when you only want the mime type.
+ * 
+ * Return value: The mime type, or NULL if there is an error reading 
+ * the file.
+ *
+ * Since: 2.14
+ **/
 char *
-_gnome_vfs_get_slow_mime_type (const char *text_uri)
+gnome_vfs_get_slow_mime_type (const char *text_uri)
+{
+	GnomeVFSResult result;
+	char *ret;
+
+	result = _gnome_vfs_get_slow_mime_type_internal (text_uri, &ret);
+	g_assert (result == GNOME_VFS_OK || ret == NULL);
+
+	return ret;
+}
+
+/* This is private due to the feature freeze, maybe it should be public */
+GnomeVFSResult
+_gnome_vfs_get_slow_mime_type_internal (const char  *text_uri,
+					char       **mime_type)
 {
 	GnomeVFSFileInfo *info;
-	char *mime_type;
 	GnomeVFSResult result;
 
+	g_assert (mime_type != NULL);
+
 	info = gnome_vfs_file_info_new ();
 	result = gnome_vfs_get_file_info (text_uri, info,
 					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 					  GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
 					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-	if (info->mime_type == NULL || result != GNOME_VFS_OK) {
-		mime_type = NULL;
-	} else {
-		mime_type = g_strdup (info->mime_type);
+	if (result == GNOME_VFS_OK && info->mime_type == NULL) {
+		result = GNOME_VFS_ERROR_GENERIC;
 	}
+
+	if (result != GNOME_VFS_OK) {
+		*mime_type = NULL;
+	} else {
+		*mime_type = g_strdup (info->mime_type);
+	}
+
 	gnome_vfs_file_info_unref (info);
 
-	return mime_type;
+	return result;
 }
 
 void
