@@ -317,6 +317,18 @@ _gnome_vfs_volume_monitor_find_volume_by_hal_udi (GnomeVFSVolumeMonitor *volume_
 			break;
 		}
 	}
+
+	/* burn:/// and cdda:// optical discs are by the hal backend added as VFS_MOUNT */
+	if (ret == NULL) {
+		for (l = volume_monitor->priv->vfs_volumes; l != NULL; l = l->next) {
+			vol = l->data;
+			if (vol->priv != NULL && vol->priv->hal_drive_udi != NULL && 
+			    strcmp (vol->priv->hal_udi, hal_udi) == 0) {
+				ret = vol;
+				break;
+			}
+		}
+	}
 	
 	return ret;
 }
@@ -361,6 +373,18 @@ _gnome_vfs_volume_monitor_find_volume_by_hal_drive_udi (GnomeVFSVolumeMonitor *v
 			break;
 		}
 	}
+
+	/* burn:/// and cdda:// optical discs are by the hal backend added as VFS_MOUNT */
+	if (ret == NULL) {
+		for (l = volume_monitor->priv->vfs_volumes; l != NULL; l = l->next) {
+			vol = l->data;
+			if (vol->priv != NULL && vol->priv->hal_drive_udi != NULL && 
+			    strcmp (vol->priv->hal_drive_udi, hal_drive_udi) == 0) {
+				ret = vol;
+				break;
+			}
+		}
+	}
 	
 	return ret;
 }
@@ -809,19 +833,19 @@ volume_name_is_unique (GnomeVFSVolumeMonitor *volume_monitor,
 
 	for (l = volume_monitor->priv->mtab_volumes; l != NULL; l = l->next) {
 		volume = l->data;
-		if (strcmp (volume->priv->display_name, name) == 0) {
+		if (volume->priv->is_user_visible && strcmp (volume->priv->display_name, name) == 0) {
 			return FALSE;
 		}
 	}
 	for (l = volume_monitor->priv->server_volumes; l != NULL; l = l->next) {
 		volume = l->data;
-		if (strcmp (volume->priv->display_name, name) == 0) {
+		if (volume->priv->is_user_visible && strcmp (volume->priv->display_name, name) == 0) {
 			return FALSE;
 		}
 	}
 	for (l = volume_monitor->priv->vfs_volumes; l != NULL; l = l->next) {
 		volume = l->data;
-		if (strcmp (volume->priv->display_name, name) == 0) {
+		if (volume->priv->is_user_visible && strcmp (volume->priv->display_name, name) == 0) {
 			return FALSE;
 		}
 	}
@@ -857,13 +881,13 @@ drive_name_is_unique (GnomeVFSVolumeMonitor *volume_monitor,
 
 	for (l = volume_monitor->priv->fstab_drives; l != NULL; l = l->next) {
 		drive = l->data;
-		if (strcmp (drive->priv->display_name, name) == 0) {
+		if (drive->priv->is_user_visible && strcmp (drive->priv->display_name, name) == 0) {
 			return FALSE;
 		}
 	}
 	for (l = volume_monitor->priv->vfs_drives; l != NULL; l = l->next) {
 		drive = l->data;
-		if (strcmp (drive->priv->display_name, name) == 0) {
+		if (drive->priv->is_user_visible && strcmp (drive->priv->display_name, name) == 0) {
 			return FALSE;
 		}
 	}
